
//----------------------------------------------------------------------------------------------------------------------------------------------------

					// Pinbelegung PIC 18F2520

//			     RB7   RB6    RB5  RB4  RB3    RB2  RB1   RB0   VDD  VSS  RC7  RC6  RC5  RC4
//			| + ###    ###     ###   ###   ###    ###   ###    ###    ###   ###   ###   ###   ###   ###  +  |
//			|     28       27       26     25      24      23     22       21      20     19     18      17     16     15         |
//			| +-+ | 				 				                                                         			   |
//			         | 		 			                                                                                             		   |
//			| +-+ |								                                                         			   |
//			|       1          2        3        4        5         6         7        8        9      10      11     12     13     14        | 
//			| + ###     ###    ###    ###   ###    ###    ###    ###   ###   ###   ###   ###   ###   ###  + |
//			    MCLR  RA0   RA1   RA2   RA3   RA4   RA5   VSS  OS1  OS2  RC0  RC1  RC2  RC3

//----------------------------------------------------------------------------------------------------------------------------------------------------

/*
	Darstellung der Ziffern 

	Ziffer	    Ports			Segmente	        HEX		     BIN	    	DEZ	 	Zuordnung
	   0	RB/0/1/2/3/4/5		a/b/c/d/e/f		0x3F	        00111111	   63                     a = RB0
	   1	RB/1/2				b/c				0x06		00000110        06                      b = RB1
	   2	RB/0/1/3/4/6	 	        a/b/d/e/g			0x5B		01011011	   91                      c = RB2
	   3	RB/0/1/2/3/6		        a/b/c/d/g			0x4F		01001111	   79                      d = RB3
	   4	RB/1/2/5/6			b/c/f/g			0x66		01100110      102                      e = RB4
	   5	RB/0/2/3/5/6		        a/c/d/f/g			0x6D		01101101      109                       f = RB5
	   6	RB/0/2/3/4/5/6		a/c/d/e/f/g		0x7D		01111101      125                      g = RB6
	   7	RB/0/1/2				a/b/c			0x07		00000111        07
	   8	RB/0/1/2/3/4/5/6	        a/b/c/d/e/f/g	        0x7F		01111111      127
	   9	RB/0/1/2/3/5/6		a/b/c/d/f/g		0x6F		01101111      111
	   
PORTB:	         0 > Segment a  		     AAAAA	   
			 1 > Segment b		  F            B	  
			 2 > Segment c		  F            B	  
			 3 > Segment d		   GGGGG  	    
			 4 > Segment e		  E            C	 
			 5 > Segment f		  E            C	  
			 6 > Segment g		   DDDDD  
			 7 > Segment P  =	 Punkt nicht geschalten, immer aus (High active)


PORTA:		RA0 = S5 (Minus)
			RA1 = S4 (Start/Stop)
			RA2 = S3 (Plus)
			RA3 = S2 (Menue)
			RA4 = TOCKI (Eingang externer Takt = 3,2768 MHz)
			RA5 = Summer
	
		 	RA0 - RA4 = Eingang 
		 	RA5 = Ausgang

PORTC:		RC0 = 7-Segmentanzeige Stunden     - ZEHNER		=	LATC = 0x3E;
			RC1 = 7-Segmentanzeige Stunden     - EINER		=	LATC = 0x3D;
			RC2 = 7-Segmentanzeige Minuten      - ZEHNER		=	LATC = 0x3B;
			RC3 = 7-Segmentanzeige Minuten      - EINER		=	LATC = 0x37;
			RC4 = 7-Segmentanzeige Sekunden  - ZEHNER	=	LATC = 0x2F;
			RC5 = 7-Segmentanzeige Sekunden  - EINER		=	LATC = 0x1F;

			RC0 - RC5 = Ausgang
*/

//20MHz Quarz = 20 000 000 Takte des PIC = 5 MHz interner Takt

//Delay10TCYx(10);		//Zeitschleife von   10 x 10 Takte
//Delay1KTCYx(10);	 	//Zeitschleife von   10 x 1000 Takte
//Delay1KTCYx(20);	 	//Zeitschleife von   20 x 1000 Takte 
//Delay10KTCYx(10);		//Zeitschleife von   10 x 10000 Takte
//Delay10KTCYx(25);		//Zeitschleife von 	 25 x 10000 Takte
//Delay10KTCYx(50);		//Zeitschleife von   50 x 10000 Takte

//-----------------------------------------------------------------------------------------------------------------------------------------------

#include 	<p18cxxx.h>			     /* Headerdatei fr PIC automatisch einbinden,
						         		 Festlegung erfolgt im Projekt */
								  
#include 	<delays.h>			     	//Headerdatei Zeitschleifen
#include		<stdio.h>
#include 	<stdlib.h>

#pragma config WDT = OFF, OSC = HS,  IESO = OFF, LVP = OFF, MCLRE = ON, STVREN = OFF, DEBUG = OFF, WRT0 = OFF, WRT1 = OFF, CP0 = OFF, CP1 = OFF, CPD = OFF, BOREN = OFF, PBADEN = OFF

#define Delay1TCY()	 Nop()			//1 Takt nichts machen

#define SUMMER_EIN   LATAbits.LATA5=1;
#define SUMMER_AUS LATAbits.LATA5=0;

//---------------------------------------------------------------------------------------------------------------------------------------------------

#pragma code _LOW_INTERRUPT_VECTOR = 0x18

#pragma interruptlow InterruptHandler

void _low_ISR (void)

		{
   		 	_asm
       			  goto InterruptHandler       // Sprung zur Interruptroutine
    			_endasm
		}
//---------------------------------------------------------------------------------------------------------------------------------------------------
#pragma code				  	 /* Nun kommt das eigentliche Programm.
					              		Da es in den Programmspeicher geschrieben 
					              		werden soll, mu zuerst die #pragma code - 
					              		Direktive stehen. */	 		   
//----------------------------------------------------------------------------------------------------------------------------------------------------

						//char = Zahlenbereich von -128  - 127
						//unsigned char = Zahlenbereich von 0 - 255

//char SEKE=0;							//Zhler fr Arrays SEKUNDEN EINER

char SEKZ=0;							//Zhler fr Arrays SEKUNDEN ZEHNER

char MINE=8;							//Zhler fr Arrays MINUTEN EINER	

char MINZ=0;							//Zhler fr Arrays MINUTEN ZEHNER

char STDE=9;							//Zhler fr Arrays STUNDEN EINER

char STDZ=1;							//Zhler fr Arrays STUNDEN ZEHNER

char ZWZSEK=0;							//Zwischenzhler SEKUNDEN

char ZWZMINE=0;						//Zwischenzhler MINUTEN EINER

char ZWZSTDZ=0;						//Zwischenzhler STUNDEN ZEHNER	

char WMINE=0;							//Speicher Weckzeit MINUTEN EINER

char WMINZ=0;							//Speicher Weckzeit MINUTEN ZEHNER

char WSTDE=0;							//Speicher Weckzeit STUNDEN EINER

char WSTDZ=0;							//Speicher Weckzeit STUNDEN ZEHNER

unsigned char i=0;

char PLUS=1;

char MENUE=1;

//char counter = 0;							//berlaufzhler Timer0 mit Startwert 0

volatile char SEKE=0;              		//Zhler fr Arrays SEKUNDEN EINER
volatile char counter = 0;                  //berlaufzhler Timer0 mit Startwert 0
//---------------------------------------------------------------------------------------------------------------------------------------------------

char ZAHLEN_array[10] = {0x3F,0x06,0x5B,0x4F,0x66,0x6D,0x7D, 0x07,0x7F,0x6F}; 	//Bitmuster Ziffern 0 - 9

					    //	    0        1        2        3       4       5        6         7       8        9	= 	Ziffer auf Anzeige

									// Array deklarieren und gleich initialisieren  	

//---------------------------------------------------------Timer 0 Initialisierung-------------------------------------------------------------

void InterruptHandler (void)

{		
				
		if(INTCONbits.TMR0IF==1)			//Wenn Timer0-Interruptflag gesetzt,
		{		

				counter++;				//dann erhhe berlaufzhler um 1.			
				
				
		INTCONbits.TMR0IF=0;			//Interruptflag lschen	
		}			
	
}
//---------------------------------------------------------------------------------------------------------------------------------------------------

void t0_ini (void)
{
			T0CONbits.TMR0ON=0;
								// Datenblatt Seite 125
	T0CON	= 0b01110111;		
		             // 1------------	Timer0 = ON
			     // -1-----------	Timer0 = 8bit (0 - 255)
		             // ---1---------	Takt an T0CKI (RA4) fr Timer0 verwenden
		             // -----1-------	Bei externem Takt an steigender Flanke inkrementieren
		             // -------0-----	Prescaler fr Timer0 verwenden
		             // ---------111	Prescaler 1:256

			T0CONbits.TMR0ON=1;

								// Datenblatt Seite 95
	INTCON	= 0b11100100; 
			     // 1------------	Enables all unmasked interrupts
			     // --1----------	Enables all unmasked peripheral interrupts
			     //----1---------	Enables the TMR0 overflow interrupt
			     //------0-------	Disables the INT0 external interrupt
			     //--------0-----	Disables the RB port change interrupt
			     //----------1---	TMR0 register has overflowed (must be cleared in software)
			     //-----------0--	The INT0 external interrupt occurred (must be cleared in software)
			     //-------------0	None of the RB<7:4> pins have changed state	
			INTCON2bits.TMR0IP=0;
	RCON = 0b00000000; 

	//T1CON	= 0b00000000;
	//T2CON	= 0b00000000;				
}
//----------------------------------------------------------------------------------------------------------------------------------------------------
void setup (void)

{
	ADCON0  	= 	0x00;			// Pins als digital I/O 
	ADCON1  	= 	0x0F;			// Pins als digital I/O 

	TRISA      	=	0x1F;			// PortA : RA0 - RA4 = Eingang / RA5 = Ausgang
	TRISB       	=	0x00;			// PortB  = Ausgang
	TRISC      	=	0x00;			// PortC = Ausgang

	
	PORTB    	= 	0x00;			// Port B lschen
	PORTC 		= 	0xFF;			// Port C Anzeigen AUS
	LATAbits.LATA5 = 0;				//Summer AUS
}

//---------------------------------------------------------------------------------------------------------------------------------------------------
void main(void)					//Hauptprogramm

{	
		setup();	
		t0_ini();	
//---------------------------------------------------------------------------------------------------------------------------------------------------
	while(1)							//Beginn Endlosschleife

	{		
			/*
				if(counter==50)			//Wenn der berlaufzhler den Wert 50 hat,

						{	
							SEKE++;											
							counter=0;		//berlaufzhler zurcksetzen
										
						}
							
			*/															
								SEKE++;								
							

			for( i = 0; i < 81; i++)			// die Schleife luft 81 mal durch
			{
				Delay10TCYx(60);		
				Nop();
			
//----------------------------------------------------------Sekunden EINER------------------------------------------------------------------		
				if (SEKE==10)

				{
					SEKE=0;
					SEKZ++;
				}

				LATB  =  ZAHLEN_array[SEKE];			//Wert Array SEKE an PortB bergeben und
				LATC  =  0x1F;							//auf Segment Sekunden EINER ausgeben
				Delay1KTCYx(11);

//----------------------------------------------------------Sekunden ZEHNER---------------------------------------------------------------	
							if (SEKZ==6)

							{
								SEKZ =0;
								MINE++;
							}

				LATB  =  ZAHLEN_array[SEKZ];			//Wert Array SEKZ an PortB bergeben und
				LATC  =  0x2F;							//auf Segment Sekunden ZEHNER ausgeben
				Delay1KTCYx(10);

//----------------------------------------------------------Minuten EINER---------------------------------------------------------------------	
										if (MINE==10)

										{
											MINE=0;
											MINZ++;	
										}

				LATB  =  ZAHLEN_array[MINE];				//Wert Array MINE an PortB bergeben und
				LATC  =  0x37;							//auf Segment Minuten EINER ausgeben
				Delay1KTCYx(10);

//----------------------------------------------------------Minuten ZEHNER------------------------------------------------------------------	
				if (MINZ==6)

				{
					MINZ=0;	
					STDE++;						
				}

				LATB  =  ZAHLEN_array[MINZ];				//Wert Array MINZ an PortB bergeben und
				LATC  =  0x3B;							//Segment Minuten ZEHNER einschalten	
				Delay1KTCYx(10);

//----------------------------------------------------------Stunden EINER--------------------------------------------------------------------	
							if (STDE==10)

							{							
								STDE=0;
								STDZ++;							
							}
	
				LATB  =  ZAHLEN_array[STDE];			//Wert Array STDE an PortB bergeben und
				LATC  =  0x3D;							//Segment Stunden EINER einschalten	
				Delay1KTCYx(10);

//----------------------------------------------------------Stunden ZEHNER-----------------------------------------------------------------	
										if ((STDE==4) && (STDZ==2))

										{	
											STDE=0;						
											STDZ=0;							
										}
			
				LATB  =  ZAHLEN_array[STDZ];			//Wert Array STDZ an PortB bergeben und
				LATC  =  0x3E;							//Segment Stunden ZEHNER einschalten
				Delay1KTCYx(10);

//----------------------------------------------------------Sekunden auf Null stellen------------------------------------------------------
			if (PORTAbits.RA1==0)
						{
							SEKE=0;					//Zhler fr Arrays SEKUNDEN EINER
							SEKZ=0;					//Zhler fr Arrays SEKUNDEN ZEHNER	
						}

//----------------------------------------------------------Minuten stellen (hochzhlen)---------------------------------------------------
			if ((PLUS==1) && (PORTAbits.RA2==0))
						{
							MINE++;											
							PLUS=0;
						}
					
			if(PORTAbits.RA2==1)
						{
							PLUS=1;
						}

//----------------------------------------------------------Stunden stellen (hochzhlen)--------------------------------------------------
			if ((MENUE==1) && (PORTAbits.RA0==0))
						{
							STDE++;											
							MENUE=0;
						}
					
			if(PORTAbits.RA0==1)
						{
							MENUE=1;
						}
		
//---------------------------------------------------------------------------------------------------------------------------------------------------
		}								
														
	}
							

}